import { promiseWithResolvers } from '../common/promiseWithResolvers.js';
export { IterableChannel };
class IterableChannel {
    #isClosed = false;
    #nextIteration = promiseWithResolvers();
    iterable = {
        [Symbol.asyncIterator]: () => ({
            next: () => this.#nextIteration.promise,
        }),
    };
    put(value) {
        if (!this.#isClosed) {
            this.#nextIteration.resolve({ done: false, value });
            this.#nextIteration = promiseWithResolvers();
        }
    }
    close() {
        this.#isClosed = true;
        this.#nextIteration.resolve({ done: true, value: undefined });
    }
}
//# sourceMappingURL=IterableChannel.js.map