import { useRef, useMemo, useEffect } from 'react';
import { useLatest } from '../common/hooks/useLatest.js';
import { isAsyncIter } from '../common/isAsyncIter.js';
import { useSimpleRerender } from '../common/hooks/useSimpleRerender.js';
export { useAsyncIterable };
function useAsyncIterable(input, initialValue) {
    const rerender = useSimpleRerender();
    const stateRef = useRef({
        value: initialValue,
        pendingFirst: true,
        done: false,
        error: undefined,
    });
    const latestInputRef = useLatest(input);
    if (!isAsyncIter(latestInputRef.current)) {
        useMemo(() => { }, [undefined]);
        useEffect(() => { }, [undefined]);
        return (stateRef.current = {
            value: latestInputRef.current,
            pendingFirst: false,
            done: false,
            error: undefined,
        });
    }
    else {
        useMemo(() => {
            stateRef.current = {
                value: stateRef.current.value,
                pendingFirst: true,
                done: false,
                error: undefined,
            };
        }, [latestInputRef.current]);
        useEffect(() => {
            const iterator = latestInputRef.current[Symbol.asyncIterator]();
            let iteratorClosedAbruptly = false;
            (async () => {
                try {
                    for await (const value of { [Symbol.asyncIterator]: () => iterator }) {
                        if (!iteratorClosedAbruptly) {
                            stateRef.current = {
                                value,
                                pendingFirst: false,
                                done: false,
                                error: undefined,
                            };
                            rerender();
                        }
                    }
                    if (!iteratorClosedAbruptly) {
                        stateRef.current = {
                            value: stateRef.current.value,
                            pendingFirst: false,
                            done: true,
                            error: undefined,
                        };
                        rerender();
                    }
                }
                catch (err) {
                    if (!iteratorClosedAbruptly) {
                        stateRef.current = {
                            value: stateRef.current.value,
                            pendingFirst: false,
                            done: true,
                            error: err,
                        };
                        rerender();
                    }
                }
            })();
            return () => {
                iteratorClosedAbruptly = true;
                iterator.return?.();
            };
        }, [latestInputRef.current]);
        return stateRef.current;
    }
}
//# sourceMappingURL=index.js.map